/************************************************************************
 *
 *  StyleConverter.java
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *  Copyright: 2002-2015 by Henrik Just
 *
 *  All Rights Reserved.
 * 
 *  Version 1.6 (2015-06-15)
 *
 */

package writer2latex.xhtml.style;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import writer2latex.office.*;
import writer2latex.util.*;
import writer2latex.xhtml.Converter;
import writer2latex.xhtml.Parser;
import writer2latex.xhtml.StyleInfo;
import writer2latex.xhtml.XhtmlConfig;
import writer2latex.xhtml.style.properties.Properties;

/** This class converts OpenDocument styles to CSS2 styles.
 * Note that some elements in OpenDocument has attributes that also maps to CSS2 properties.
 * Example: the width of a text box.
 * Also note, that some OpenDocument style properties cannot be mapped to CSS2 without creating an additional inline element.
 * The class uses one helper class per OpenDocument style family (paragraph, frame etc.)
 */
public class Styles extends Parser {

    // Helpers for text styles
    private TextStyleParser textStyleParser;
    private ParStyleParser parStyleParser;
    private HeadingStyleParser headingStyleParser;
    private ListStyleParser listStyleParser;
    private SectionStyleParser sectionStyleParser;

    // Helpers for table styles
    private TableStyleParser tableStyleParser;
    private RowStyleParser rowStyleParser;
    private CellStyleParser cellStyleParser;

    // Helpers for drawing styles
    private FrameStyleParser frameStyleParser;
    private PresentationStyleParser presentationStyleParser;
	
    // Helper for page styles
    private PageStyleParser pageStyleParser;
    
    /** Create a new <code>StyleConverter</code>
     * 
     * @param ofr the office reader used to access the source document
     * @param config the configuration to use
     * @param converter the main converter
     * @param nType the XHTML type
     */
    public Styles(OfficeReader ofr, XhtmlConfig config, Converter converter, int nType) {
        super(ofr,config,converter);
        // Create the helpers
        textStyleParser = new TextStyleParser(ofr,config,converter,nType);
        parStyleParser = new ParStyleParser(ofr,config,converter,nType);
        headingStyleParser = new HeadingStyleParser(ofr,config,converter,nType);
        listStyleParser = new ListStyleParser(ofr,config,converter,nType);
        sectionStyleParser = new SectionStyleParser(ofr,config,converter,nType);
        tableStyleParser = new TableStyleParser(ofr,config,converter,nType);
        rowStyleParser = new RowStyleParser(ofr,config,converter,nType);
        cellStyleParser = new CellStyleParser(ofr,config,converter,nType);
        frameStyleParser = new FrameStyleParser(ofr,config,converter,nType);
        presentationStyleParser = new PresentationStyleParser(ofr,config,converter,nType);
        pageStyleParser = new PageStyleParser(ofr,config,converter,nType);
    }
	
    // Accessor methods for helpers: We need to override the style helper accessors
    
    public TextStyleParser getTextSP() { return textStyleParser; }

    public ParStyleParser getParSP() { return parStyleParser; }

    public HeadingStyleParser getHeadingSc() { return headingStyleParser; }

    public ListStyleParser getListSc() { return listStyleParser; }

    public SectionStyleParser getSectionSc() { return sectionStyleParser; }

    public TableStyleParser getTableSc() { return tableStyleParser; }

    public RowStyleParser getRowSc() { return rowStyleParser; }

    public CellStyleParser getCellSc() { return cellStyleParser; }

    public FrameStyleParser getFrameSc() { return frameStyleParser; }

    public PresentationStyleParser getPresentationSc() { return presentationStyleParser; }

    public PageStyleParser getPageSc() { return pageStyleParser; }
	
    /** Apply the default language of the source document on an XHTML element
     * 
     * @param node the XHTML element
     */
    public void applyDefaultLanguage(Element node) {
        StyleWithProperties style = getDefaultStyle();
        if (style!=null) {
            StyleInfo info = new StyleInfo();
            StyleParser.applyLang(style,info);
            writeStyle(info,node);
        }
    }
    
    /** Export style information as a string of plain CSS code
     * 
     * @param bIndent true if the CSS code should be indented
     * @return the CSS code
     */
    public String allStyleSelectors(boolean bIndent) {
    	String sIndent = bIndent ? "      " : "";
        StringBuilder buf = new StringBuilder();
        
        exportDefaultStyle(buf,sIndent);
		
        // Export declarations from helpers
        // For OpenDocument documents created with OOo only some will generate content:
        //   Text documents: text, par, list, frame
        //   Spreadsheet documents: cell
        //   Presentation documents: frame, presentation, page
        buf.append(textStyleParser.composeStyleDeclarations());
        buf.append(parStyleParser.composeStyleDeclarations());
        buf.append(headingStyleParser.composeStyleDeclarations());
        buf.append(listStyleParser.composeStyleDeclarations());
        buf.append(sectionStyleParser.composeStyleDeclarations());
        buf.append(cellStyleParser.composeStyleDeclarations());
        buf.append(tableStyleParser.composeStyleDeclarations());
        buf.append(rowStyleParser.composeStyleDeclarations());
        buf.append(frameStyleParser.composeStyleDeclarations());
        buf.append(presentationStyleParser.composeStyleDeclarations());
        buf.append(pageStyleParser.composeStyleDeclarations());
        return buf.toString();
    }
    
    /** Export style information as an XHTML style element
     * 
     * @param htmlDOM the XHTML DOM to which the generated element belongs
     * @return the style element
     */
    public Node getStylesNode(Document htmlDOM) {
        String styleSelectors = allStyleSelectors(config.prettyPrint());
		
        // Create node
        if (styleSelectors.length()>0) {
            Element styleElement = htmlDOM.createElement("style");
            styleElement.setAttribute("media","all");
            styleElement.setAttribute("type","text/css");
            styleElement.appendChild(htmlDOM.createTextNode(config.prettyPrint() ? "\n" : " "));
            styleElement.appendChild(htmlDOM.createTextNode(styleSelectors));
            if (config.prettyPrint()) { styleElement.appendChild(htmlDOM.createTextNode("    ")); }
            return styleElement;
        }
        else {
            return null;
        }
    }
    
    // Private helper methods
    
    private void exportDefaultStyle(StringBuilder buf, String sIndent) {
        // Export default style
        if (config.xhtmlCustomStylesheet().length()==0 &&
            (config.xhtmlFormatting()==XhtmlConfig.CONVERT_ALL ||
            config.xhtmlFormatting()==XhtmlConfig.IGNORE_HARD)) {
        	Properties props = new Properties(";");
        
            // Default paragraph/cell/frame style is applied to the body element
            StyleWithProperties defaultStyle = getDefaultStyle();
            if (defaultStyle!=null) {
                // text properties only!
                getTextSP().cssTextCommon(defaultStyle,props,true);
                if (config.useDefaultFont() && config.defaultFontName().length()>0) {
                	props.addProperty("font-family", "'"+config.defaultFontName()+"'");
                }
            }
            
            // For text documents (XHTML only), also set maximum width
            if (ofr.isText() && !converter.isOPS()) {
            	String sMaxWidth = config.getMaxWidth().trim();
            	if (sMaxWidth.length()>0) {
		            props.addProperty("max-width", sMaxWidth);
		            props.addProperty("margin-left","auto");
		            props.addProperty("margin-right","auto");
            	}
            }
            
            // Apply properties to body
            if (!props.isEmpty()) {
	            buf.append(sIndent);
	            buf.append("body {");
	            buf.append(props.toString());
	            buf.append("}");
	            buf.append(config.prettyPrint() ? "\n" : " ");
            }
        }
    }
	
    private StyleWithProperties getDefaultStyle() {
        if (ofr.isSpreadsheet()) return ofr.getDefaultCellStyle();
        else if (ofr.isPresentation()) return ofr.getDefaultFrameStyle();
        else return ofr.getDefaultParStyle();
    }

}